module Hermes
  module RexClient
    class SyncStructuresTask

      def sync(options = {})
        date = options[:date]
        client = options[:client] || Hermes::RexClient::Client.new
        yopmail = options[:yopmail] || true, 
        dont_update_parent = options[:dont_update_parent] || false
        dont_update_lessor = options[:dont_update_lessor] || false
        dont_update_entity = options[:dont_update_entity] || false

        start = Time.now

        rex_last_updated = date || Setting.get_settings_for('rexclient').values['rex_last_updated']

        puts '-' * 100
        puts 'Structures lastModifiedDate after date'
        puts '-' * 100
        client.traverse_structures({'traceability.lastModifiedDate' => ">#{date}"},{ 
          dont_update_lessor: dont_update_lessor, 
          dont_update_entity: dont_update_entity
          }, Proc.new do |rex_entity| 

            className = (rex_entity['idEntityDefinition'] == client.ed_entity) ? 'Hermes::Entity' : 'Hermes::Lessor'
            line = "Started : Syncro #{className} id_cristal #{rex_entity['fields']['id_cristal']}"
            if client.logger
              client.logger.info line
            end 
            puts(line)

            if rex_entity['fields']['id_absorbant_cristal'].blank?
              Hermes::RexClient::UpdateStructureTask.new.update_entity(rex_entity, className, client, dont_update_parent)
            end

            line = "Ended : Syncro #{className} id_cristal #{rex_entity['fields']['id_cristal']}"
            if client.logger
              client.logger.info line
            end 
            puts(line)

          end
        )

        puts '-' * 100
        puts 'Contacts lastModifiedDate after date'
        puts '-' * 100
        client.traverse_contacts({'traceability.lastModifiedDate' => ">#{date}"},{}, Proc.new do |rex_entity| 

            line = "Started : Syncro Contact id_cristal #{rex_entity['fields']['id_cristal']}"
            if client.logger
              client.logger.info line
            end 
            puts(line)

            Hermes::RexClient::UpdateContactTask.new.update_contact(rex_entity, client, yopmail, dont_update_lessor, dont_update_entity)

            line = "Ended : Syncro Contact id_cristal #{rex_entity['fields']['id_cristal']}"
            if client.logger
              client.logger.info line
            end 

            puts(line)
          end
        )

        finish = Time.now

        diff = finish - start


        line = "REX sync took : #{diff} sec"
        if client.logger
          client.logger.info line
        end 
      end
    end
  end
end
